#ifndef DUNE_MICROSTRUCTURE_VOIGTHELPER_HH
#define DUNE_MICROSTRUCTURE_VOIGTHELPER_HH

/** \file
 * \brief Various helper functions related to Voigt notation
 */

#include <dune/common/fvector.hh>
#include <dune/common/fmatrix.hh>


template<typename T, int dim>
using VoigtVector = Dune::FieldVector<T,dim*(dim+1)/2>;

template<typename T, int dim>
using VoigtMatrix = Dune::FieldMatrix<T,dim*(dim+1)/2,dim*(dim+1)/2>;


template<typename T>
VoigtVector<T,3> matrixToVoigt(const Dune::FieldMatrix<T,3,3>& matrix)
{
  return {matrix[0][0], matrix[1][1], matrix[2][2], 2.0*matrix[1][2], 2.0*matrix[0][2], 2.0*matrix[0][1]};
}

template<typename T>
Dune::FieldMatrix<T,3,3> voigtToMatrix(const VoigtVector<T,3>& v)
{
  return {{v[0], v[5]/2.0, v[4]/2.0}, {v[5]/2.0, v[1], v[3]/2.0}, {v[4]/2.0, v[3]/2.0, v[2]}};
}

/** \brief Interpret Voigt vectors as symmetric matrices and compute their Frobenius scalar voigtScalarProduct
 */
template<typename T>
T voigtScalarProduct(const VoigtVector<T,3>& a, const VoigtVector<T,3>& b)
{
  return a[0]*b[0] + a[1]*b[1] + a[2]*b[2] + 2*a[3]*b[3] + 2*a[4]*b[4] + 2*a[5]*b[5];
}

/** \brief Return the symmetric part of a matrix, in Voigt notation */
template<typename T>
VoigtVector<T,3> symVoigt(const Dune::FieldMatrix<T,3,3>& matrix)
{
  VoigtVector<T,3> result = {matrix[0][0],
                             matrix[1][1],
                             matrix[2][2],
                             // The 0.5 is part of the sym operator, the 2.0 is from the conversion to Voigt notation
                             2 * 0.5 * (matrix[1][2] + matrix[2][1]),
                             2 * 0.5 * (matrix[0][2] + matrix[2][0]),
                             2 * 0.5 * (matrix[0][1] + matrix[1][0])};

  return result;
}

#endif   // DUNE_MICROSTRUCTURE_VOIGTHELPER_HH
